// This file is under GNU General Public License 3.0
// see LICENSE.txt

#pragma once

#include "Sync_impl.h"

#ifdef __cplusplus
extern "C" {
#endif

// state machine for KeySync

// states

typedef enum _KeySync_state {
    KeySync_state_None = None,
    KeySync_state_Init = Init,
    Sole, 
    HandshakingOfferer, 
    HandshakingRequester, 
    HandshakingPhase1Offerer, 
    HandshakingPhase1Requester, 
    HandshakingPhase2Offerer, 
    HandshakingPhase2Requester, 
    FormingGroupOfferer, 
    FormingGroupRequester, 
    Grouped, 
    HandshakingToJoin, 
    HandshakingToJoinPhase1, 
    HandshakingToJoinPhase2, 
    JoiningGroup, 
    HandshakingGrouped, 
    HandshakingGroupedPhase1, 
    HandshakingGroupedPhase2
} KeySync_state;

// events

typedef enum _KeySync_event {
    KeySync_event_None = None,
    KeySync_event_Init = Init,
    Beacon = 2,
    NegotiationRequest = 3,
    NegotiationOpen = 4,
    Rollback = 5,
    CommitReject = 6,
    CommitAcceptOfferer = 7,
    CommitAcceptRequester = 8,
    CommitAccept = 9,
    CommitAcceptForGroup = 10,
    GroupTrustThisKey = 11,
    GroupKeys = 12,
    OwnKeysOfferer = 13,
    OwnKeysRequester = 14,
    KeySync_event_Extra = Extra,
    Accept = 129,
    Reject = 130,
    Cancel = 131,
    KeyGen, 
    CannotDecrypt
} KeySync_event;

// state machine

#ifndef NDEBUG
const char *KeySync_state_name(int state);
const char *KeySync_event_name(int event);
#endif

// the state machine function is returning the next state in case of a
// transition or None for staying

KeySync_state fsm_KeySync(
        PEP_SESSION session,
        KeySync_state state,
        KeySync_event event
    );

#ifdef __cplusplus
}
#endif

