// This file is under GNU General Public License 3.0
// see LICENSE.txt

#include "platform.h"

#include "pEp_internal.h"
#include "Sync_event.h"
#include "Sync_func.h"
#include "KeySync_fsm.h"

Sync_event_t *new_Sync_event(SYNC_PR fsm, int event, Sync_t *msg)
{
    Sync_event_t *ev = (Sync_event_t *) calloc(1, sizeof(Sync_event_t));
    assert(ev);
    if (!ev)
        return NULL;

    ev->fsm = fsm;
    ev->event = event;
    ev->msg = msg;

    if (msg) {
        switch (fsm) {
            case Sync_PR_keysync: {
                switch (msg->choice.keysync.present) {
                    case KeySync_PR_beacon:
                        ev->event = Beacon;
                        break;
                    case KeySync_PR_negotiationRequest:
                        ev->event = NegotiationRequest;
                        break;
                    case KeySync_PR_negotiationOpen:
                        ev->event = NegotiationOpen;
                        break;
                    case KeySync_PR_rollback:
                        ev->event = Rollback;
                        break;
                    case KeySync_PR_commitReject:
                        ev->event = CommitReject;
                        break;
                    case KeySync_PR_commitAcceptOfferer:
                        ev->event = CommitAcceptOfferer;
                        break;
                    case KeySync_PR_commitAcceptRequester:
                        ev->event = CommitAcceptRequester;
                        break;
                    case KeySync_PR_commitAccept:
                        ev->event = CommitAccept;
                        break;
                    case KeySync_PR_commitAcceptForGroup:
                        ev->event = CommitAcceptForGroup;
                        break;
                    case KeySync_PR_groupTrustThisKey:
                        ev->event = GroupTrustThisKey;
                        break;
                    case KeySync_PR_groupKeys:
                        ev->event = GroupKeys;
                        break;
                    case KeySync_PR_ownKeysOfferer:
                        ev->event = OwnKeysOfferer;
                        break;
                    case KeySync_PR_ownKeysRequester:
                        ev->event = OwnKeysRequester;
                        break;
                    default:
                        // unknown message type
                        free(ev);
                        return NULL;
                }
                break;
            }
            
            default:
                // unknown protocol
                free(ev);
                return NULL;
        }
    }

    return ev;
}

void free_Sync_event(Sync_event_t *ev)
{
    if (ev) {
        free_identity_list(ev->own_identities);
        free_Sync_message(ev->msg);
        free_identity(ev->from);
        free(ev->sender_fpr);
        free(ev);
    }
}

