// This file is under GNU General Public License 3.0
// see LICENSE.txt

#pragma once

#include "pEpEngine.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct Sync SYNC;
typedef int SYNC_PR;

typedef struct Sync_event {
    // state machine data
    SYNC_PR fsm;
    int event;
    SYNC *msg;

    // transport data
    pEp_identity *from;
    char *sender_fpr;

    identity_list *own_identities;
} Sync_event_t;


// new_Sync_event() - allocate a new Sync_event
//
//  parameters:
//      fsm (in)        finite state machine the event is for
//      event (in)      event or None
//      msg (in)        message to compute event from
//
//  return value:
//      pointer to new event or NULL in case of failure
//
//  caveat:
//      event must be valid for fsm or None
//      in case msg is given event will be calculated out of message

Sync_event_t *new_Sync_event(SYNC_PR fsm, int event, SYNC *msg);

#define SYNC_TIMEOUT_EVENT new_Sync_event(Sync_PR_NOTHING, 0, NULL);


// free_Sync_event() - free memory occupied by event
//
//  parameters:
//      ev (in)         event to free

void free_Sync_event(Sync_event_t *ev);


#ifdef __cplusplus
}
#endif

