// This file is under GNU General Public License 3.0
// see LICENSE.txt

#pragma once

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>

#include "../asn.1/Sync.h"

// state

struct Sync_state_s {
    // own state

    struct own_Sync_state_s {
        stringlist_t *keys;
        identity_list *identities;

        // TIDs we're using ourselves
        TID_t challenge;
        TID_t negotiation;
    } own;

    // state we learned about our communication partner

    struct comm_partner_state_s {
        // transport data we expect
        char *sender_fpr;

        // TIDs our comm partner wants to have
        TID_t challenge;
        TID_t negotiation;
    } comm_partner;

    // input buffer for actual transport data coming in

    struct transport_data_s {
        // transport data we got
        pEp_identity *from;
        char *sender_fpr;
    } transport;
    
    // input/output buffer for KeySync messages
    
    struct _KeySync_state_s {
        int state;
    
        TID_t challenge;
        TID_t negotiation;
        bool is_group;
        Hash_t key;
        IdentityList_t ownIdentities;
    } keysync;
};

void free_Sync_state(PEP_SESSION session);

// functions for protocol Sync

Sync_t *new_Sync_message(Sync_PR fsm, int message_type);
void free_Sync_message(Sync_t *msg);

PEP_STATUS update_Sync_state(PEP_SESSION session, Sync_t *msg,
        Sync_PR *fsm, int *message_type);

PEP_STATUS update_Sync_message(PEP_SESSION session, Sync_t *msg);

#ifdef __cplusplus
}
#endif

