// This file is under GNU General Public License 3.0
// see LICENSE.txt

#pragma once

#include "fsm_common.h"
#include "Sync_event.h"
#include "message_api.h"
#include "../asn.1/Sync.h"

#define SYNC_THRESHOLD 10
#define KEYSYNC_THRESHOLD 300

#ifdef __cplusplus
extern "C" {
#endif

// conditions

PEP_STATUS deviceGrouped(PEP_SESSION session, bool *result);
PEP_STATUS sameChallenge(PEP_SESSION session, bool *result);
PEP_STATUS weAreOfferer(PEP_SESSION session, bool *result);
PEP_STATUS sameNegotiation(PEP_SESSION session, bool *result);
PEP_STATUS partnerIsGrouped(PEP_SESSION session, bool *result);
PEP_STATUS sameNegotiationAndPartner(PEP_SESSION session, bool *result);

// actions

PEP_STATUS newChallengeAndNegotiationBase(PEP_SESSION session);
PEP_STATUS showBeingSole(PEP_SESSION session);
PEP_STATUS useOwnChallenge(PEP_SESSION session);
PEP_STATUS openNegotiation(PEP_SESSION session);
PEP_STATUS tellWeAreNotGrouped(PEP_SESSION session);
PEP_STATUS storeNegotiation(PEP_SESSION session);
PEP_STATUS showSoleHandshake(PEP_SESSION session);
PEP_STATUS disable(PEP_SESSION session);
PEP_STATUS trustThisKey(PEP_SESSION session);
PEP_STATUS untrustThisKey(PEP_SESSION session);
PEP_STATUS prepareOwnKeys(PEP_SESSION session);
PEP_STATUS showFormingGroup(PEP_SESSION session);
PEP_STATUS saveGroupKeys(PEP_SESSION session);
PEP_STATUS receivedKeysAreDefaultKeys(PEP_SESSION session);
PEP_STATUS showGroupCreated(PEP_SESSION session);
PEP_STATUS ownKeysAreDefaultKeys(PEP_SESSION session);
PEP_STATUS showBeingInGroup(PEP_SESSION session);
PEP_STATUS tellWeAreGrouped(PEP_SESSION session);
PEP_STATUS showJoinGroupHandshake(PEP_SESSION session);
PEP_STATUS showDeviceAdded(PEP_SESSION session);
PEP_STATUS showGroupedHandshake(PEP_SESSION session);
PEP_STATUS hideHandshakeDialog(PEP_SESSION session);

// timeout handler

PEP_STATUS KeySyncTimeoutHandler(PEP_SESSION session);

// send message about an event to communication partners using state

PEP_STATUS send_Sync_message(
        PEP_SESSION session, 
        Sync_PR fsm,
        int message_type
    );

// receive message and store it in state

PEP_STATUS recv_Sync_event(
        PEP_SESSION session,
        Sync_event_t *ev
    );

// state machine driver
// if fsm or event set to 0 use fields in src if present

PEP_STATUS Sync_driver(
        PEP_SESSION session,
        Sync_PR fsm,
        int event
    );

// API being used by the engine internally

// call this if you need to signal an external event
// caveat: the ownership of own_identities goes to the callee

PEP_STATUS signal_Sync_event(
        PEP_SESSION session, 
        Sync_PR fsm,
        int event,
        identity_list *own_identities
    );

// call this if you are a transport and are receiving
// a Sync message

PEP_STATUS signal_Sync_message(
        PEP_SESSION session, 
        PEP_rating rating,
        const char *data,
        size_t size,
        const pEp_identity *from,
        const char *sender_fpr
    );

#ifdef __cplusplus
}
#endif

