// This file is under GNU General Public License 3.0
// see LICENSE.txt

#pragma once


#include "pEpEngine.h"


#ifdef __cplusplus
extern "C" {
#endif


struct Sync;

// decode_Sync_message() - decode PER encoded Sync message
//
//  parameters:
//      data (in)               PER encoded data
//      size (in)               size of PER encoded data
//      msg (out)               Sync message
//
//  caveat:
//      msg goes into the ownership of the caller

DYNAMIC_API PEP_STATUS decode_Sync_message(
        const char *data,
        size_t size,
        struct Sync **msg
    );


// encode_Sync_message() - encode Sync message into PER encoded data
//
//  parameters:
//      msg (in)                Sync message
//      data (out)              PER encoded data
//      size (out)              size of PER encoded data
//
//  caveat:
//      data goes to the ownership of the caller

DYNAMIC_API PEP_STATUS encode_Sync_message(
        struct Sync *msg,
        char **data,
        size_t *size
    );


// PER_to_XER_Sync_msg() - decode Sync message from PER into XER
//
//  parameters:
//      data (in)               PER encoded data
//      size (in)               size of PER encoded data
//      text (out)              XER text of the same Sync message

DYNAMIC_API PEP_STATUS PER_to_XER_Sync_msg(
        const char *data,
        size_t size,
        char **text
    );


// XER_to_PER_Sync_msg() - encode Sync message from XER into PER
//
//  parameters:
//      text (in)               string with XER text of the Sync message
//      data (out)              PER encoded data
//      size (out)              size of PER encoded data

DYNAMIC_API PEP_STATUS XER_to_PER_Sync_msg(
        const char *text,
        char **data,
        size_t *size
    );


#ifdef __cplusplus
}
#endif

